package com.imageresizer;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.drawable.Drawable;
import android.util.Log;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.bumptech.glide.gifdecoder.StandardGifDecoder;
import com.bumptech.glide.load.resource.gif.GifDrawable;
import com.bumptech.glide.request.target.CustomTarget;
import com.bumptech.glide.request.transition.Transition;
import com.gifencoder.AnimatedGifEncoder;
import com.n4no.webpencoder.codec.WebpBitmapEncoder;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

public class TargetGif extends CustomTarget<GifDrawable> {
    private final String TAG = TargetGif.class.getCanonicalName();
    public final StaticOptions options;
    public CodecCallback callback;
    public Context context;
    private final ExecutorService executor = Executors.newSingleThreadExecutor(); // Create an Executor

    private final String extentions = "webp";

    public TargetGif(Context context, CodecCallback callback, StaticOptions options) {
        this.context = context;
        this.callback = callback;
        this.options = options;
    }
    public TargetGif(StaticOptions options){
        this.options = options;
    }

    @Override
    public void onResourceReady(@NonNull GifDrawable resource, @Nullable Transition<? super GifDrawable> transition) {
        // 0.
        StandardGifDecoder decoder = null;
        try {
            decoder = DecoderProvider.getGifDecoder(resource);
        } catch (IllegalAccessException | NoSuchFieldException e) {
            Log.d(TAG,"StandardGifDecoder : "+e.getMessage());
            throw new RuntimeException(e);
        }

        StandardGifDecoder finalDecoder = decoder;
        if (finalDecoder == null) {
            Log.d(TAG,"finalDecoder null ");
            throw new IllegalArgumentException("GIF decoder is null");
        }

        // background
        executor.submit(()->{
            String path = null;
            Log.d(TAG,"outFomrat "+options.outFormat);
            if(options.outFormat.equalsIgnoreCase("webp")){
                Log.d(TAG,"encodeToWebp");
                path = encodeToWebp(finalDecoder);
            }else{
                Log.d(TAG,"encodeToGif");
                path = encodeToGif(finalDecoder);
            }
            callback.onComplete(path);
        });



    }

    public String encodeToGif(StandardGifDecoder finalDecoder) {
        // 1. create File
        File outputFile =  new FileUtility().createOutputFile("gif");

        // 2. Encoder
        GifEncoder gifEncoder = new GifEncoder(outputFile, finalDecoder.getDelay(0));

        // 3. Loop
        for (int i = 0; i < finalDecoder.getFrameCount(); i++) {
            Log.d(TAG,"i "+i+"/"+finalDecoder.getFrameCount());

            Bitmap bitmap = finalDecoder.getNextFrame();
            if (bitmap == null) {
                throw new RuntimeException("Null Bitmap Frame by getNextFrame ");
            }

            Bitmap newBitmap = BitmapUtility.resizeBitmap(bitmap, options);
            bitmap.recycle();

            gifEncoder.addFrame(newBitmap, finalDecoder.getNextDelay());
            newBitmap.recycle();

            // Move Next frame
            finalDecoder.advance();
        }
        gifEncoder.finish();
        return outputFile.getAbsolutePath();
    }

    public String encodeToGif2(StandardGifDecoder finalDecoder) {
        // 1. create File
        File outputFile =  new FileUtility().createOutputFile("gif");

        // 2. Encoder
        AnimatedGifEncoder gifEncoder = new AnimatedGifEncoder();
        gifEncoder.start(outputFile.getAbsolutePath());

        // delay ;
        int count = finalDecoder.getFrameCount();

        int delay = finalDecoder.getNextDelay();
        if(delay < 80) delay = 80;
        gifEncoder.setDelay(delay);

        // 3. Loop
        for (int i = 0; i < count; i++) {
            Log.d(TAG,"i "+i+"/"+count);

            Bitmap bitmap = finalDecoder.getNextFrame();
            if (bitmap == null) {
                gifEncoder.finish();
                throw new RuntimeException("Null Bitmap Frame by getNextFrame ");
            }

            Bitmap newBitmap = BitmapUtility.resizeBitmap(bitmap, options);

            gifEncoder.addFrame(newBitmap);
            bitmap.recycle();
            newBitmap.recycle();
//            options.width, options.height
            // Move Next frame
            finalDecoder.advance();
        }
        gifEncoder.finish();
        // finall close and inform
        return outputFile.getAbsolutePath();
    }


    // backuo
    public String encodeToWebp(StandardGifDecoder finalDecoder) {

            Log.d(TAG,"Encoding to webo");
            // 1. create File
            File outputFile =  new FileUtility().createOutputFile("webp");

            // 2. Encoder
            WebpBitmapEncoder encoder;
            try {
                encoder = new WebpBitmapEncoder(outputFile);
            } catch (FileNotFoundException e) {
                Log.d(TAG,"Error created file not found: "+e.getMessage());
                throw new RuntimeException(e);
            }
            encoder.setLoops(0);


            // 3. Loop
            int count = finalDecoder.getFrameCount();
            for (int i = 0; i < count; i++) {
                Log.d(TAG,"'i"+i+"/"+count);
                int delay = finalDecoder.getNextDelay();
                if(delay < 80) delay = 80;

                Bitmap bitmap = finalDecoder.getNextFrame();
                if (bitmap == null) {
                    throw new RuntimeException("Null Bitmap Frame by getNextFrame ");
                }
                try {
                    Bitmap newBitmap = BitmapUtility.resizeBitmap(bitmap, options);
                    encoder.setDuration(delay);
                    encoder.writeFrame(newBitmap, options.quality);
                    bitmap.recycle();
                    newBitmap.recycle();
                    if (i == count - 1) encoder.close();
                } catch (IOException e) {

                    Log.d(TAG,"e "+e.getMessage());
                    throw new RuntimeException(e);
                }

                // Move Next frame
                finalDecoder.advance();
            }

            // finall close and inform
            return outputFile.getAbsolutePath();

    }

    @Override
    public void onLoadCleared(@Nullable Drawable placeholder) {

    }
}
