import { NativeModules, Platform } from 'react-native';

const LINKING_ERROR =
  `The package 'react-native-image-resizer' doesn't seem to be linked. Make sure: \n\n` +
  Platform.select({ ios: "- You have run 'pod install'\n", default: '' }) +
  '- You rebuilt the app after installing the package\n' +
  '- You are not using Expo Go\n';

// @ts-expect-error
const isTurboModuleEnabled = global.__turboModuleProxy != null;

const ImageResizerModule = isTurboModuleEnabled
  ? require('./NativeImageResizer').default
  : NativeModules.ImageResizer;

const ImageResizer = ImageResizerModule
  ? ImageResizerModule
  : new Proxy(
      {},
      {
        get() {
          throw new Error(LINKING_ERROR);
        },
      }
    );

interface StaticOptions {
  width?: number;
  height?: number;
  outFormat?: string;
  mode?: string;
  quality?: number;
}

export function staticImage(uri:string, userOptions: StaticOptions): Promise<void> {
  return new Promise((resolve, reject) => {
    
    if(typeof uri !== 'string'){
      throw new Error("You must provider uri:string of image file as 1st arguement");
    }

    ImageResizer.Static(uri, userOptions)
      .then((r) => {
        resolve(r);
      })
      .catch((error) => {
        reject(error);
      });
  });
}
export function animatedImage(uri:string, userOptions: StaticOptions): Promise<void> {
  return new Promise((resolve, reject) => {
    
    if(typeof uri !== 'string'){
      throw new Error("You must provider uri:string of image file as 1st arguement");
    }
    
    ImageResizer.Animated(uri, userOptions)
      .then((r) => {
        resolve(r);
      })
      .catch((error) => {
        reject(error);
      });
  });
}

export function staticImageBatch(uris:string[], userOptions: StaticOptions): Promise<void> {
  return new Promise((resolve, reject) => {
    
    if (!uris || uris.length === 0) {
      reject("Image URIs array is empty");
      return;
    }

    ImageResizer.StaticBatch(uris, userOptions)
      .then((r) => {
        resolve(r);
      })
      .catch((error) => {
        reject(error);
      });
  });
}

export function animatedImageBatch(uris:string[], userOptions: StaticOptions): Promise<void> {
  return new Promise((resolve, reject) => {
    
    if (!uris || uris.length === 0) {
      reject("Image URIs array is empty");
      return;
    }
    
    ImageResizer.AnimatedBatch(uris, userOptions)
      .then((r) => {
        resolve(r);
      })
      .catch((error) => {
        reject(error);
      });
  });
}

// 

const RNImageResizer = {
  still : {  
    single: staticImage, 
    batch: staticImageBatch, 
  },
  anim :  {  
    single: animatedImage,
    batch: animatedImageBatch
  },
  Scale : ImageResizer.getConstants().Scale,
  Format : ImageResizer.getConstants().Format
}

export default RNImageResizer;
// export default ImageResizer;
